@@ -1,17 +1,16 @@
-from typing import Optional
-import os
-import re
-import json
+import os, re, json, tempfile
 import numpy as np
 import pandas as pd
-import tempfile
-from collections import defaultdict
-from typing import Tuple, Dict
+from collections import namedtuple
+from typing import Dict, Tuple, Optional
 
 from alto.utils import prefix_float, run_command
-from .bcl_utils import lane_manager, path_is_flowcell, transfer_flowcell
+from .bcl_utils import lane_manager, path_is_bcl, transfer_flowcell
+from .fastq_utils import sample_manager, path_is_fastq, transfer_fastq
 
 
+FlowcellType = namedtuple('FlowcellType', ['type', 'manager'])
+
 
 def read_wdl_inputs(input_json: str) -> dict:
     """Load inputs from either a JSON file or a JSON string.
@@ -68,9 +67,9 @@ def transfer_data(
     dest: str,
     backend: str,
     dry_run: bool,
-    flowcells: Dict[str, lane_manager] = None,
-    verbose: bool = True,
+    flowcells: Dict[str, FlowcellType] = None,
     profile: Optional[str] = None,
+    verbose: bool = True,
 ) -> None:
     """Transfer source to dest (cloud destination).
        backend, choosing from gcp and aws.
@@ -79,16 +78,29 @@ def transfer_data(
     if verbose:
         print(f'{"Dry run: " if dry_run else ""}Uploading {source} to {dest}.')
 
-    if path_is_flowcell(source):
-        lanes = flowcells[source].get_lanes() if flowcells is not None else ['*']
-        transfer_flowcell(
-            source=source,
-            dest=dest,
-            backend=backend,
-            lanes=lanes,
-            dry_run=dry_run,
-            profile=profile,
-        )
+    if flowcells != None and source in flowcells:
+        flowcell = flowcells[source]
+        if flowcell.type == 'bcl':
+            transfer_flowcell(
+                source=source,
+                dest=dest,
+                backend=backend,
+                lanes=flowcell.manager.get_lanes(),
+                dry_run=dry_run,
+                profile=profile,
+                verbose=verbose,
+            )
+        else:
+            assert flowcell.type == 'fastq'
+            transfer_fastq(
+                source=source,
+                dest=dest,
+                backend=backend,
+                samples=flowcell.manager.get_samples(),
+                dry_run=dry_run,
+                profile=profile,
+                verbose=verbose,
+            )
     else:
         if os.path.isdir(source):
             strato_cmd = ['strato', 'sync', '--backend', backend, '--ionice', '-m', '--quiet', source, dest]
@@ -106,8 +118,8 @@ def transfer_sample_sheet(
     input_file_to_output_url: dict,
     url_gen: cloud_url_factory,
     dry_run: bool,
-    verbose: bool = True,
     profile: Optional[str] = None,
+    verbose: bool = True,
 ) -> Tuple[str, bool]:
     """Check sample sheet and upload files inside it.
        input_file: sample sheet
@@ -122,36 +134,74 @@ def transfer_sample_sheet(
     """
     is_changed = False
 
+    # Terminate if no access to sample sheet.
+    if not os.access(input_file, os.R_OK):
+        raise PermissionError(f"Need read access to '{input_file}'!")
+
+    # If cannot process, upload its original content.
     try:
         df = pd.read_csv(input_file, sep=None, engine='python', header=None, index_col=False)
     except Exception:
         return input_file, is_changed
 
-    flowcells = defaultdict(lane_manager)
+    flowcells = {}
     col_names = np.char.array(df.iloc[0,:], unicode = True).lower()
 
-    if ('flowcell' in col_names) and ('lane' in col_names):
+    if ('flowcell' in col_names) or ('location' in col_names):
+        flowcell_keyword = 'flowcell' if 'flowcell' in col_names else 'location'
         df.columns = col_names
-        for idx, row in df[1:].iterrows():
-            flowcells[row['flowcell']].update_lanes(row['lane'])
 
-    for idx, row in df[1:].iterrows():
+        sample_keyword = None
+        if 'library' in col_names:
+            sample_keyword = 'library'
+        elif 'sample' in col_names:
+            sample_keyword = 'sample'
+        else:
+            raise ValueError("Cannot detect either Library or Sample column in the sample sheet!")
+
+        for _, row in df[1:].iterrows():
+            path = os.path.abspath(row[flowcell_keyword])
+            if not os.path.isdir(path):
+                raise ValueError(f"{path} is not a folder!")
+            if not os.access(path, os.X_OK):
+                raise PermissionError(f"Need execution access to folder '{path}'!")
+
+            flowcell = None
+            if path in flowcells:
+                flowcell = flowcells[path]
+            else:
+                if path_is_bcl(path):
+                    flowcell = FlowcellType(type='bcl', manager=lane_manager())
+                elif path_is_fastq(path):
+                    flowcell = FlowcellType(type='fastq', manager=sample_manager())
+                else:
+                    raise ValueError(f"{path} is neither a BCL folder nor a FASTQ folder!")
+                flowcells[path] = flowcell
+
+            if flowcell.type == 'bcl':
+                flowcell.manager.update_lanes(row['lane'] if 'lane' in row else '*')
+            else:
+                flowcell.manager.update_samples(row[sample_keyword])
+
+    for _, row in df[1:].iterrows():
         for idxc, value in row.iteritems():
             if isinstance(value, str) and os.path.exists(value):
-                value = os.path.abspath(value)
-                sub_url = input_file_to_output_url.get(value, None)
+                source = os.path.abspath(value)
+                sub_url = input_file_to_output_url.get(source, None)
+
                 if sub_url is None:
-                    sub_url = url_gen.get_unique_url(value)
+                    sub_url = url_gen.get_unique_url(source)
                     transfer_data(
-                        source=value,
+                        source=source,
                         dest=sub_url,
                         backend=backend,
                         dry_run=dry_run,
                         flowcells=flowcells,
-                        verbose=verbose,
                         profile=profile,
+                        verbose=verbose,
                     )
-                    input_file_to_output_url[value] = sub_url
+                    input_file_to_output_url[source] = sub_url
+
                 row[idxc] = sub_url
                 is_changed = True
 
@@ -194,6 +244,10 @@ def upload_to_cloud_bucket(
         Path for the JSON file storing updated inputs.
     dry_run: `bool`
         If dry run, only print commands but do not execute.
+    verbose: `bool`, default: ``True``
+        If print out the underlying upload commands on screen.
+    profile: `str`, default: ``None``
+        For AWS backend only, it's used for specifying a non-default AWS profile.
 
     Returns
     -------
@@ -201,7 +255,7 @@ def upload_to_cloud_bucket(
 
     Examples
     --------
-    >>> upload_to_cloud_bucket(inputs, 'gcp', 'my_bucket', 'input_files', 'updated_inputs.json', False)
+    >>> upload_to_cloud_bucket(inputs, 'gcp', 'my_bucket', 'input_files', 'updated_inputs.json')
     """
     if bucket_folder is not None:
         bucket += f"/{bucket_folder.strip('/')}"
@@ -242,6 +296,7 @@ def upload_to_cloud_bucket(
                 verbose=verbose,
                 profile=profile,
             )
+
             inputs[k] = input_url
             if is_changed: # delete temporary file after uploading
                 os.remove(input_path)
